#pragma once
#include <Color4.hpp>
#include <Colors.hpp>

#include "JGL/JGL.h"
#include "JGL/types/Font.h"

namespace JUI {

    struct TextStyle {
        Color4 color;
        JGL::Font font;
        int size;
    };

    /// Defines interface for widgets with textual elements.
    /// Any object that renders text will use these methods to apply style.
    class TextStyler {
    public:
        virtual ~TextStyler() = default;
    public:
#pragma region Getters
        /// @returns the font of this widget.
        virtual JGL::Font Font() const;
        /// @return The text-point-size of this widget.
        virtual int TextSize() const;
        /// @return The text-color of this widget. @see class Color4.
        virtual Color4 TextColor() const;
#pragma endregion
    public:
#pragma region Setters
        virtual void Font(const JGL::Font& value);
        virtual void TextSize(int value);
        virtual void TextColor(const Color4& color);

        virtual void Style(const JGL::Font& font, int size, const Color4& color);


#pragma endregion
    protected:
        JGL::Font font = Fonts::Jupiteroid;
        int text_size = 12;
        Color4 text_color = Colors::White;
    };
}
