/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Checkbox.hpp
/// @desc It's a checkbox.
/// @edit 2024-10-28

#pragma once

#include <JUI/Base/Widget.hpp>
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Mixins/Clickable.hpp>
#include <JUI/Mixins/Hoverable.hpp>
#include <JUI/Widgets/Button.hpp>
#include <JUI/Base/ImageBase.hpp>
#include <JUI/Widgets/ImageButton.hpp>

namespace JUI {
    // TODO: Find a nice way to implement a checkmark.

    class CheckboxBase {
    public:
        [[nodiscard]] Color4 CheckedColor() const { return check_color; }
        void CheckedColor(const Color4& color) { check_color = color; }
        bool IsChecked() const { return checked;}
        void SetChecked(bool value) { checked = value; }
        Tween* TweenCheckedColor(const Color4& goal, TweenInfo info = {});
    protected:
        bool checked = false;
        Color4 check_color = Style::Checkbox::CheckmarkColor;
        Color4 uncheck_color = Style::BackgroundColor;
    };

    class Checkbox : public Button, public CheckboxBase {
    public:
        Checkbox();
        explicit Checkbox(Widget *parent);
        void Update(float delta) override;
        void OnRelease(const J3ML::LinearAlgebra::Vector2 &mouse_pos, const JUI::MouseButton &bnt, bool still_hovering) override;
        void InnerDraw() override;

    protected:
    private:
    };

    class ImageCheckbox : public ImageButton, public CheckboxBase {
    public:
    protected:
    private:
    };

}