#pragma once
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/ImageRect.hpp>
#include <JUI/Widgets/FpsGraph.hpp>

#include "Window.hpp"


namespace JUI {

    ///
    struct DataPoint {
        Vector2 pos;
        Color4 color;
    };

    ///
    class FpsGraph : public JUI::ImageRect
    {
    public:

        /// Plots all stored data-points onto the render target.
        void RenderDataPoints();

        /// The default constructor initializes the basic layout of the FpsGraph class. Member variables are also zero-initialized.
        FpsGraph();

        /// Constructs an FpsGraph by explicitly specifying its parent element.
        explicit FpsGraph(Widget* parent);

        /// Inserts a new data point into the graph's data set.
        void Plot(const Vector2& pt, const Color4& col);
        void Plot(const DataPoint& data_pt);

        /// Performs update logic on the state and appearance of this graph widget.
        void Update(float delta) override;

        /// A builtin preset for style and layout that appears docked at the bottom of the screen. It effectively behaves as an overlaid frame-graph.
        /// TODO: Better name.
        void SetupAsPseudoDockedElementAtBottomOfScreen();

        void ShowTargetFPS(bool show);

        bool ShowTargetFPS() const;

        void TargetFPS(float fps);

        float TargetFPS() const;

        std::vector<DataPoint> data;
        JGL::RenderTarget* canvas;


        /// If the current FPS is lower than this value, use high_fps_color.
        float HighFPSRange() const;

        /// If the current FPS is lower than this value, and higher than SubparFPSRange(), use target_fps_color.
        float TargetFPSRange() const;

        /// If the current FPS is lower than this value, and higher than LowFPSRange(), use subpar_fps_color.
        float SubparFPSRange() const;

        /// If the current FPS is lower than this value, and higher than UnplayableFPSRange()
        float LowFPSRange() const;

        float UnplayableFPSRange() const;

        /// Any FPS lower than this value will use slideshow_fps_color.
        float SlideshowFPSRange() const;

    protected:
        float graph_height = 50;
        float graph_width = 500;
        float sample_history = 500;
        float y_axis_scale = 600;
        float target_fps = 60.f;
        bool show_target_fps_bar = true;
        Color4 target_fps_bar_color = Colors::White;
        Color4 target_fps_label_color = Colors::Black;


        Color4 high_fps_color = Colors::Blue;
        Color4 target_fps_color = Colors::Green;
        Color4 subpar_fps_color = Colors::Yellow;
        Color4 low_fps_color = Colors::Red;
        Color4 unplayable_fps_color = Colors::Purples::Magenta;
        Color4 slideshow_fps_color = Colors::Black;

        float high_fps_range_factor = 2.f;
        float target_fps_range_factor = 1.f;
        float subpar_fps_range_factor = 0.5f;
        float low_fps_range_factor = 0.25f;
        float unplayable_fps_range_factor = 0.125f;
        float slideshow_fps_range_factor = 0.0625f;
    private:
    };

    class FpsGraphWindow : public Window
    {
    public:
        FpsGraphWindow();

        explicit FpsGraphWindow(Widget* parent);
        FpsGraph* GraphWidget() {return data_graph; }
        void DockGraph() {
            this->Close();
            // TODO:: unsafe.
            data_graph->Parent(this->GetParent());
            data_graph->SetupAsPseudoDockedElementAtBottomOfScreen();
        }
        void UndockGraph() {
            this->Open();
            data_graph->Parent(this->Content());
        }
    protected:
        FpsGraph* data_graph;
    private:
    };
}
