/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file ListLayout.hpp
/// @desc Clickable Button widget.
/// @edit 2024-08-02

#pragma once

#include <JUI/Base/LayoutContainer.hpp>

namespace JUI
{
    class ListLayout : public LayoutContainer
    {
    public:
        ListLayout();
        explicit ListLayout(Widget* parent);
        ~ListLayout() override {}
    };

    namespace LayoutOrder
    {
        enum class H { LEFT, RIGHT };
        enum class V { TOP, BOTTOM };
    }

    /// Lays child elements out in a vertical list.
    /// Child element positions are overridden by this widget.
    class VerticalListLayout : public ListLayout {
    public:
        VerticalListLayout();
        explicit VerticalListLayout(Widget* parent);

        /// Computes the order and positioning of all child elements to display them in an ordered list.
        void ApplyLayout() override;
        void LayoutOrder(LayoutOrder::V order);
        [[nodiscard]] LayoutOrder::V LayoutOrder() const;

        /// @return the most-recently-computed sum of the heights of all child elements, with padding also taken into account.
        [[nodiscard]] float CurrentContentHeight() const;
    protected:
        LayoutOrder::V layout = LayoutOrder::V::TOP;
        float content_height = 0;
    };

    /// Lays child elements out in a horizontal list.
    /// Child element positions are overridden by this widget.
    class HorizontalListLayout : public ListLayout {
    public:
        HorizontalListLayout();
        explicit HorizontalListLayout(Widget* parent);
        void ApplyLayout() override;
        void LayoutOrder(LayoutOrder::H order);
        [[nodiscard]] LayoutOrder::H LayoutOrder() const;
        [[nodiscard]] float CurrentContentWidth() const;
    protected:
        LayoutOrder::H layout = LayoutOrder::H::LEFT;
        float content_width = 0;
    };
}