/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2025 Redacted Software
/// This work is dedicated to the public domain.

/// @file Collapsible.cpp
/// @desc A rectangular widget with a
/// @edit 2025-02-14
#pragma once

#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/Widgets/TextButton.hpp>

// TODO: Each "Tab" needs its own content box.
namespace JUI {
    class TabPageView : public Rect {
    public:

        /// Constructs
        TabPageView()
        {

        }
        explicit TabPageView(Widget* parent);

        /// Adds a tab-page-set to the TabView by creating a default-style TextButton with the specified content.
        std::pair<TextButton*, Rect*> AddTabPageSet(const std::string& content);
        /// Adds a tab-page-set to the TabView by explicitly specifying the TextButton 'tab' and Rect 'page'.
        Widget* AddTabPage(TextButton* tab_element);


        int GetActiveTabIndex();
        void SetActiveTabIndex(int index);

        Rect* GetTabsContainer();
        Rect* GetPagesContainer();
        HorizontalListLayout* GetTabsListLayout();


        TextButton* GetTabByIndex(int index);
        Rect* GetPageByIndex(int index);


        void NextTab();
        void PrevTab();
        void FirstTab();
        void LastTab();
    protected:
        std::vector<TextButton*> tabs;
        std::vector<Rect*> pages;

        Rect* tabs_container;

        Rect* page_container;

    private:
    };
}