/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Widget.hpp
/// @desc Base Widget Class - All JUI widgets extend their behavior from this class.
/// @edit 2024-10-11

#pragma once
#include "ContextMenu.hpp"

namespace JUI
{
    class UtilityBar;


    /// A horizontal toolbar widget that is designed to be used at the top of windows, and subwindow widgets.
    /// Convenience functions are provided for adding tool buttons and sub-menus.
    /// @see class ContextMenu
    class UtilityBar : public Rect, public NestableMenu, public TextStyler
    {
    public:

        UtilityBar();

        explicit UtilityBar(Widget* parent);

        /// Adds a new 'Submenu', which consists of a labeled button, which opens a contextual-submenu
        ContextMenu* AddSubmenu(const std::string& name) override;

        TextButton* AddButton(const std::string& name) override;
        TextButton* AddButton(const std::string& name, const std::function<void()>& callback) override;

        Separator* AddSeparator(const UDim& size = 5_px) override;


        std::vector<TextButton*> GetButtons() { return buttons;}
        std::vector<ContextMenu*> GetSubmenus() { return submenus;}

        void Font(const JGL::Font &value) override;
        void TextSize(int value) override;
        void TextColor(const Color4 &color) override;

    protected:
        std::vector<TextButton*> buttons;
        std::vector<ContextMenu*> submenus;
        HorizontalListLayout* layout = nullptr;

    private:
    };
}
