#include <JUI/Tween.hpp>
#include <J3ML/J3ML.hpp>

namespace JUI
{
    using namespace J3ML;

    float EasingFunctions::EaseInSine(float t) { return Math::Sin(Math::PiOverTwo * t); }

    float EasingFunctions::EaseOutSine(float t) { return 1 + Math::Sin(Math::PiOverTwo * (--t)); }

    float EasingFunctions::EaseInOutSine(float t) { return 0.5f * (1 + Math::Sin(Math::Pi * (t - 0.5f))); }

    float EasingFunctions::EaseInQuad(float t) { return t * t;}

    float EasingFunctions::EaseOutQuad(float t) { return t * (2.f - t);}

    float EasingFunctions::EaseInOutQuad(float t) { return t < 0.5f ? 2.f * t * t : t * (4.f - 2.f * t) - 1; }

    float EasingFunctions::EaseInCubic(float t) { return t * t * t;}

    float EasingFunctions::EaseOutCubic(float t) { return 1.f + (--t) * t * t; }

    float EasingFunctions::EaseInOutCubic(float t) { return t < 0.5f ? 4.f * t * t * t : 1 + (--t) * (2 * (--t)) * (2 * t); }

    float EasingFunctions::EaseInQuart(float t) {
        t *= t;
        return t * t;
    }

    float EasingFunctions::EaseOutQuart(float t) {
        t = (--t) * t;
        return 1.f - t * t;
    }

    float EasingFunctions::EaseInOutQuart(float t) {
        if (t < 0.5f) {
            t *= t;
            return 8 * t * t;
        } else {
            t = (--t) * t;
            return 1 - 8 * t * t;
        }
    }

    float EasingFunctions::EaseInQuint(float t) {
        float t2 = t * t;
        return t * t2 * t2;
    }

    float EasingFunctions::EaseOutQuint(float t) {
        float t2 = (--t) * t;
        return 1 + t * t2 * t2;
    }

    float EasingFunctions::EaseInOutQuint(float t) {
        float t2;
        if ( t < 0.5f) {
            t2 = t * t;
            return 16 * t * t2 * t2;
        } else {
            t2 = (--t) * t;
            return 1 + 16 * t * t2 * t2;
        }
    }

    float EasingFunctions::EaseInExpo(float t) {
        return (Math::Pow(2, 8*t) - 1) / 255;
    }

    float EasingFunctions::EaseOutExpo(float t) {
        return 1 - Math::Pow(2, -8*t);
    }

    float EasingFunctions::EaseInOutExpo(float t) {
        if (t < 0.5f) {
            return (Math::Pow(2, 16 * t) - 1) / 510;
        } else {
            return 1 - 0.5f * Math::Pow(2, -16 * (t - 0.5f));
        }
    }

    float EasingFunctions::EaseInCirc(float t) {
        return 1 - Math::Sqrt(1 - t);
    }

    float EasingFunctions::EaseOutCirc(float t) {
        return Math::Sqrt(t);
    }

    float EasingFunctions::EaseInOutCirc(float t) {
        if (t < 0.5f) {
            return (1 - Math::Sqrt(1 - 2 * t)) * 0.5f;
        } else {
            return (1 + Math::Sqrt(2 * t - 1)) * 0.5f;
        }
    }

    float EasingFunctions::EaseInBack(float t) { return t * t * (2.70158f * t - 1.70158f); }

    float EasingFunctions::EaseOutBack(float t) { return 1 + (--t) * t * (2.70158 * t + 1.70158); }

    float EasingFunctions::EaseInOutBack(float t) {
        if (t < 0.5f) {
            return t * t * (7 * t - 2.5f) * 2.f;
        } else {
            return 1 + (--t) * t * 2 * (7 * t + 2.5f);
        }
    }

    float EasingFunctions::EaseInElastic(float t) {
        float t2 = t * t;
        return t2 * t2 * Math::Sin(t * Math::Pi * 4.5f);
    }

    float EasingFunctions::EaseOutElastic(float t) {
        float t2 = (t - 1) * (t - 1);
        return 1 - t2 * t2 * Math::Cos(t * Math::Pi * 4.5f);
    }

    float EasingFunctions::EaseInOutElastic(float t) {
        float t2;
        if (t < 0.45f) {
            t2 = t * t;
            return 8 * t2 * t2 * Math::Sin(t * Math::Pi * 9);
        } else if (t < 0.55f) {
            return 0.5f + 0.75f * Math::Sin(t * Math::Pi * 4);
        } else {
            t2 = (t - 1) * (t - 1);
            return 1 - 8 * t2 * t2 * Math::Sin(t * Math::Pi * 9);
        }
    }

    float EasingFunctions::EaseInBounce(float t) {
        return Math::Pow(2, 6 * (t - 1)) * Math::Abs(Math::Sin(t * Math::Pi * 3.5f));
    }

    float EasingFunctions::EaseOutBounce(float t) {
        return 1 - Math::Pow(2, -6 * t ) * Math::Abs(Math::Cos(t * Math::Pi * 3.5f));
    }

    float EasingFunctions::EaseInOutBounce(float t) {
        if (t < 0.5f) {
            return 8 * Math::Pow(2, 8 * (t - 1)) * Math::Abs(Math::Sin(t * Math::Pi));
        } else {
            return 1 - 8 * Math::Pow(2, -8 * t) * Math::Abs(Math::Sin(t * Math::Pi * 7));
        }
    }

    float EasingFunctions::EaseInOutLinear(float t) { return t;}

    void Tween::Update(float elapsed) {

        if (time <= 0)
        {
            // TODO: Error in this case.
            completed = true;
        }


        if (delay_time > 0)
        {
            delay_time -= elapsed;
            return;
        }

        progress += (elapsed / time);

        if (progress >= 1.f)
        {
            progress = 1.f;
            if (!completed)
            {
                completed = true;
                Completed.Invoke();
            }
            return;
        }

        float modified_progress = easing_func(progress);

        tick_func(elapsed, modified_progress);
    }

    void Tween::ForceFinish() {
        progress = 1.f;
    }

    void Tween::Cancel() {
        completed = true;
    }

    bool Tween::Paused() const { return paused; }

    bool Tween::HasCompleted() const { return completed; }

    void Tween::Start() { progress = 0; Resume(); }

    Tween::Tween(TweenTickFunc tick_func) {
        this->tick_func = tick_func;
    }
}