#include <JUI/Widgets/ColorPicker.hpp>

std::string rgb2hex(int r, int g, int b, bool with_head) {
    std::stringstream ss;
    if (with_head)
        ss << "#";
    ss << std::hex << (r << 16 | g << 8 | b );
    return ss.str();
}

void JUI::ColorPicker::Font(const JGL::Font &value) {
    TextStyler::Font(value);
    hue_label->Font(value);
    sat_label->Font(value);
    bri_label->Font(value);
    hex_label->Font(value);
}

void JUI::ColorPicker::TextSize(int size) {
    TextStyler::TextSize(size);
    hue_label->TextSize(size);
    sat_label->TextSize(size);
    bri_label->TextSize(size);
    hex_label->TextSize(size);
}

void JUI::ColorPicker::TextColor(const Color4 &color) {
    TextStyler::TextColor(color);
    hue_label->TextColor(color);
    sat_label->TextColor(color);
    bri_label->TextColor(color);
    hex_label->TextColor(color);
}

void JUI::ColorPicker::StyleSlider(Slider *slider) {
    slider->ScrubberSize({16_px, 16_px});
    slider->CornerRounding(8);
    slider->BGColor(Colors::DarkGray);
    slider->ScrubberRounding(8);
    slider->Size({100_percent-10_px, 16_px});
}

JUI::ColorPicker::ColorPicker(): Rect() {
    Name("ColorPicker");

    Size({100_percent, 100_percent});
    //auto* row_layout = new JUI::VerticalListLayout(this);

    hue_slider = new JUI::Slider(this);
    StyleSlider(hue_slider);
    hue_slider->Position({50_percent, 4_px});
    hue_slider->AnchorPoint({0.5f, 0.f});

    hue_slider->Minimum(0.f);
    hue_slider->Maximum(360);
    hue_slider->Interval(1.f / 360.f);
    hue_slider->ValueChanged += [this] (float val) mutable {
        hue_label->Content(std::format("Hue: {}", Math::FloorInt(val)));
        hue = val;
        hue_slider->ScrubberColor(Color4::FromHSV(hue, 1.f, 1.f));
        OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));
        RecomputeVisuals();
    };

    hue_label = new JUI::Text(hue_slider);
    hue_label->AlignCenterHorizontally();
    hue_label->AlignTop();
    hue_label->TextColor(Colors::White);
    hue_label->Content("Hue: 0");

    sat_slider = new JUI::Slider(this);
    StyleSlider(sat_slider);
    sat_slider->Position({50_percent, 24_px});
    sat_slider->AnchorPoint({0.5f, 0.f});
    sat_slider->Minimum(0);
    sat_slider->Maximum(1);
    sat_slider->Interval(1e-3f);
    sat_slider->ValueChanged += [this] (float val) mutable {
        sat_label->Content(std::format("Saturation: {}%", Math::Floor(val*100.f)));

        sat = val;
        OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));

        RecomputeVisuals();
    };

    sat_label = new JUI::Text(sat_slider);
    sat_label->AlignCenterHorizontally();
    sat_label->AlignTop();
    sat_label->TextColor(Colors::White);
    sat_label->Content("Saturation: 100%");

    bri_slider = new JUI::Slider(this);
    StyleSlider(bri_slider);
    bri_slider->Position({50_percent, 44_px});
    bri_slider->AnchorPoint({0.5f, 0.f});
    bri_slider->Minimum(0);
    bri_slider->Maximum(1);
    bri_slider->Interval(1e-3f);
    bri_slider->ValueChanged += [this] (float val) mutable {
        bri_label->Content(std::format("Brightness: {}%", Math::Floor(val*100.f)));
        bri = val;
        OnColorValueChanged.Invoke(Color4::FromHSV(hue, sat, bri));
        RecomputeVisuals();
    };

    bri_label = new JUI::Text(bri_slider);
    bri_label->AlignCenterHorizontally();
    bri_label->AlignTop();
    bri_label->TextColor(Colors::White);
    bri_label->Content("Brightness: 100%");

    //auto* hue_box = new JUI::Rect();

    hex_label = new JUI::TextRect(this);
    hex_label->AlignCenterHorizontally();
    hex_label->Size({100_percent, 24_px});
    hex_label->Position({0_px, 100_percent});
    hex_label->Content("#FFZZGG");
}

JUI::ColorPicker::ColorPicker(Widget *parent): ColorPicker() {
    Parent(parent);
}

void JUI::ColorPicker::RecomputeVisuals() {
    Color4 computed = Color4::FromHSV(hue, sat, bri);

    Color4 inverse = Color4(255 - computed.r, 255 - computed.g, 255 - computed.b);

    hex_label->TextColor(inverse);
    hex_label->BGColor(computed);
    hex_label->Content(std::format("{}", rgb2hex(computed.r, computed.g, computed.b, true)));
}
