#include <JUI/Widgets/NineSlice.hpp>
#include <JGL/JGL.h>

namespace JUI {
    AABB2D JUI::NineSliceRect::TopLeftQuad() const { return top_left_quad; }

    AABB2D NineSliceRect::TopRightQuad() const { return top_right_quad; }

    AABB2D NineSliceRect::BottomLeftQuad() const { return bottom_left_quad; }

    AABB2D NineSliceRect::BottomRightQuad() const { return bottom_right_quad; }

    AABB2D NineSliceRect::TopQuad() const { return top_quad; }

    AABB2D NineSliceRect::LeftQuad() const { return left_quad; }

    AABB2D NineSliceRect::RightQuad() const { return right_quad; }

    AABB2D NineSliceRect::BottomQuad() const { return bottom_quad; }

    AABB2D NineSliceRect::CenterQuad() const { return center_quad; }

    void NineSliceRect::TopLeftQuad(const AABB2D &quad) { top_left_quad = quad; }

    void NineSliceRect::TopRightQuad(const AABB2D &quad) { top_right_quad = quad; }

    void NineSliceRect::BottomLeftQuad(const AABB2D &quad) { bottom_left_quad = quad; }

    void NineSliceRect::BottomRightQuad(const AABB2D &quad) { bottom_right_quad = quad; }

    void NineSliceRect::TopQuad(const AABB2D &quad) { top_quad = quad;}

    void NineSliceRect::RightQuad(const AABB2D &quad) { right_quad = quad;}

    void NineSliceRect::BottomQuad(const AABB2D &quad) { bottom_quad = quad;}

    void NineSliceRect::LeftQuad(const AABB2D &quad) { left_quad = quad;}

    void NineSliceRect::CenterQuad(const AABB2D &quad) { center_quad = quad;}

    void NineSliceRect::Draw() {
        Rect::Draw();

        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        // Draw Top-Left Quad.
        Vector2 tl_computed_pos = abs_pos;
        auto tl_quad = TopLeftQuad();
        JGL::J2D::DrawPartialRenderTarget(texture, tl_computed_pos, tl_quad.minPoint, tl_quad.maxPoint);

        // Draw Top-Right Quad.
        auto tr_quad = TopRightQuad();
        Vector2 tr_computed_pos = abs_pos + Vector2(abs_size.x - tr_quad.maxPoint.x, 0);
        JGL::J2D::DrawPartialRenderTarget(texture, tr_computed_pos, tr_quad.minPoint, tr_quad.maxPoint);

        // Draw Bottom Left Quad
        auto bl_quad = BottomLeftQuad();
        Vector2 bl_computed_pos = abs_pos + Vector2(0, abs_size.y - bl_quad.maxPoint.y);
        JGL::J2D::DrawPartialRenderTarget(texture, bl_computed_pos, bl_quad.minPoint, bl_quad.maxPoint);

        // Draw Bottom Right Quad
        auto br_quad = BottomRightQuad();
        Vector2 br_computed_pos = abs_pos + abs_size - br_quad.maxPoint;
        JGL::J2D::DrawPartialRenderTarget(texture, br_computed_pos, br_quad.minPoint, br_quad.maxPoint);

        // Draw Top-Quad.
        Vector2 t_computed_pos = abs_pos + Vector2(tl_quad.maxPoint.x, 0);
        auto t_quad = TopQuad();
        float abs_width_minus_corners = abs_size.x - (tl_quad.maxPoint.x + tr_quad.maxPoint.x);
        float t_scaling = abs_width_minus_corners / t_quad.maxPoint.x;
        JGL::J2D::DrawPartialRenderTarget(texture, t_computed_pos, t_quad.minPoint, t_quad.maxPoint, 0, {0,0}, {t_scaling, 1});

        // Draw Bottom Quad
        auto b_quad = BottomQuad();
        Vector2 b_computed_pos = abs_pos + Vector2(tl_quad.maxPoint.x, abs_size.y - b_quad.maxPoint.y);
        float b_scaling = abs_width_minus_corners / b_quad.maxPoint.x;
        JGL::J2D::DrawPartialRenderTarget(texture, b_computed_pos, b_quad.minPoint, b_quad.maxPoint, 0, {0,0}, {b_scaling, 1});

        // Draw Left Quad
        Vector2 l_computed_pos = abs_pos + Vector2(0, tl_quad.maxPoint.y);
        auto l_quad = LeftQuad();
        float abs_height_minus_corners = abs_size.y - (tl_quad.maxPoint.y + tr_quad.maxPoint.y);
        float l_scaling = abs_height_minus_corners / l_quad.maxPoint.y;
        JGL::J2D::DrawPartialRenderTarget(texture, l_computed_pos, l_quad.minPoint, l_quad.maxPoint, 0, {0, 0}, {1, l_scaling});

        // Draw Right Quad
        auto r_quad = RightQuad();
        Vector2 r_computed_pos = abs_pos + Vector2(abs_size.x - tr_quad.maxPoint.x, tr_quad.maxPoint.y);
        float r_scaling = abs_height_minus_corners / r_quad.maxPoint.y;
        JGL::J2D::DrawPartialRenderTarget(texture, r_computed_pos, r_quad.minPoint, r_quad.maxPoint, 0, {0,0}, {1, r_scaling});

        // Draw Center Quad
        auto c_quad = CenterQuad();
        Vector2 c_computed_pos = abs_pos + tl_quad.maxPoint;
        Vector2 c_scaling = Vector2(abs_width_minus_corners, abs_height_minus_corners) / c_quad.maxPoint;
        JGL::J2D::DrawPartialRenderTarget(texture, c_computed_pos, c_quad.minPoint, c_quad.maxPoint, 0, {0,0}, c_scaling);
    }

    NineSliceRect::NineSliceRect() : Rect(), ImageBase() {
        Name("NineSliceRect");
    }

    NineSliceRect::NineSliceRect(Widget *parent) : Rect(parent), ImageBase() {}

    NineSliceRect::NineSliceRect(Widget *parent, JGL::Texture *texture) : Rect(parent), ImageBase(texture) {}
}

